<!DOCTYPE html>
{% set target = meta.redirect_target %}
<html>
<head>
  <title>The clangd documentation has moved to clangd.llvm.org</title>
  <link rel="canonical" href="{{ target }}"/>
  <meta http-equiv="refresh" content="0;url={{ target }}" />
</head>
<body>
  <h1>The clangd documentation has moved to clangd.llvm.org</h1>
  <a href="{{ target }}">The new site<a>
  <script>window.location='{{ target }}'</script>
</body>
</html>
