// NOTE: Assertions have been autogenerated by utils/update_cc_test_checks.py UTC_ARGS: --version 4
// REQUIRES: riscv-registered-target
// RUN: %clang_cc1 -triple riscv64 -target-feature +zve64x \
// RUN:   -target-feature +zvfbfmin -disable-O0-optnone \
// RUN:   -emit-llvm %s -o - | opt -S -passes=mem2reg | \
// RUN:   FileCheck --check-prefix=CHECK-RV64 %s

#include <riscv_vector.h>

// CHECK-RV64-LABEL: define dso_local <vscale x 1 x bfloat> @test_vundefined_bf16mf4(
// CHECK-RV64-SAME: ) #[[ATTR0:[0-9]+]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret <vscale x 1 x bfloat> poison
//
vbfloat16mf4_t test_vundefined_bf16mf4() {
  return __riscv_vundefined_bf16mf4();
}

// CHECK-RV64-LABEL: define dso_local <vscale x 2 x bfloat> @test_vundefined_bf16mf2(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret <vscale x 2 x bfloat> poison
//
vbfloat16mf2_t test_vundefined_bf16mf2() {
  return __riscv_vundefined_bf16mf2();
}

// CHECK-RV64-LABEL: define dso_local <vscale x 4 x bfloat> @test_vundefined_bf16m1(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret <vscale x 4 x bfloat> poison
//
vbfloat16m1_t test_vundefined_bf16m1() { return __riscv_vundefined_bf16m1(); }

// CHECK-RV64-LABEL: define dso_local <vscale x 8 x bfloat> @test_vundefined_bf16m2(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret <vscale x 8 x bfloat> poison
//
vbfloat16m2_t test_vundefined_bf16m2() { return __riscv_vundefined_bf16m2(); }

// CHECK-RV64-LABEL: define dso_local <vscale x 16 x bfloat> @test_vundefined_bf16m4(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret <vscale x 16 x bfloat> poison
//
vbfloat16m4_t test_vundefined_bf16m4() { return __riscv_vundefined_bf16m4(); }

// CHECK-RV64-LABEL: define dso_local <vscale x 32 x bfloat> @test_vundefined_bf16m8(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret <vscale x 32 x bfloat> poison
//
vbfloat16m8_t test_vundefined_bf16m8() { return __riscv_vundefined_bf16m8(); }

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 2 x i8>, 2) @test_vundefined_bf16mf4x2(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 2 x i8>, 2) poison
//
vbfloat16mf4x2_t test_vundefined_bf16mf4x2() {
  return __riscv_vundefined_bf16mf4x2();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 2 x i8>, 3) @test_vundefined_bf16mf4x3(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 2 x i8>, 3) poison
//
vbfloat16mf4x3_t test_vundefined_bf16mf4x3() {
  return __riscv_vundefined_bf16mf4x3();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 2 x i8>, 4) @test_vundefined_bf16mf4x4(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 2 x i8>, 4) poison
//
vbfloat16mf4x4_t test_vundefined_bf16mf4x4() {
  return __riscv_vundefined_bf16mf4x4();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 2 x i8>, 5) @test_vundefined_bf16mf4x5(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 2 x i8>, 5) poison
//
vbfloat16mf4x5_t test_vundefined_bf16mf4x5() {
  return __riscv_vundefined_bf16mf4x5();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 2 x i8>, 6) @test_vundefined_bf16mf4x6(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 2 x i8>, 6) poison
//
vbfloat16mf4x6_t test_vundefined_bf16mf4x6() {
  return __riscv_vundefined_bf16mf4x6();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 2 x i8>, 7) @test_vundefined_bf16mf4x7(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 2 x i8>, 7) poison
//
vbfloat16mf4x7_t test_vundefined_bf16mf4x7() {
  return __riscv_vundefined_bf16mf4x7();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 2 x i8>, 8) @test_vundefined_bf16mf4x8(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 2 x i8>, 8) poison
//
vbfloat16mf4x8_t test_vundefined_bf16mf4x8() {
  return __riscv_vundefined_bf16mf4x8();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 4 x i8>, 2) @test_vundefined_bf16mf2x2(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 4 x i8>, 2) poison
//
vbfloat16mf2x2_t test_vundefined_bf16mf2x2() {
  return __riscv_vundefined_bf16mf2x2();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 4 x i8>, 3) @test_vundefined_bf16mf2x3(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 4 x i8>, 3) poison
//
vbfloat16mf2x3_t test_vundefined_bf16mf2x3() {
  return __riscv_vundefined_bf16mf2x3();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 4 x i8>, 4) @test_vundefined_bf16mf2x4(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 4 x i8>, 4) poison
//
vbfloat16mf2x4_t test_vundefined_bf16mf2x4() {
  return __riscv_vundefined_bf16mf2x4();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 4 x i8>, 5) @test_vundefined_bf16mf2x5(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 4 x i8>, 5) poison
//
vbfloat16mf2x5_t test_vundefined_bf16mf2x5() {
  return __riscv_vundefined_bf16mf2x5();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 4 x i8>, 6) @test_vundefined_bf16mf2x6(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 4 x i8>, 6) poison
//
vbfloat16mf2x6_t test_vundefined_bf16mf2x6() {
  return __riscv_vundefined_bf16mf2x6();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 4 x i8>, 7) @test_vundefined_bf16mf2x7(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 4 x i8>, 7) poison
//
vbfloat16mf2x7_t test_vundefined_bf16mf2x7() {
  return __riscv_vundefined_bf16mf2x7();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 4 x i8>, 8) @test_vundefined_bf16mf2x8(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 4 x i8>, 8) poison
//
vbfloat16mf2x8_t test_vundefined_bf16mf2x8() {
  return __riscv_vundefined_bf16mf2x8();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 8 x i8>, 2) @test_vundefined_bf16m1x2(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 8 x i8>, 2) poison
//
vbfloat16m1x2_t test_vundefined_bf16m1x2() {
  return __riscv_vundefined_bf16m1x2();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 8 x i8>, 3) @test_vundefined_bf16m1x3(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 8 x i8>, 3) poison
//
vbfloat16m1x3_t test_vundefined_bf16m1x3() {
  return __riscv_vundefined_bf16m1x3();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 8 x i8>, 4) @test_vundefined_bf16m1x4(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 8 x i8>, 4) poison
//
vbfloat16m1x4_t test_vundefined_bf16m1x4() {
  return __riscv_vundefined_bf16m1x4();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 8 x i8>, 5) @test_vundefined_bf16m1x5(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 8 x i8>, 5) poison
//
vbfloat16m1x5_t test_vundefined_bf16m1x5() {
  return __riscv_vundefined_bf16m1x5();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 8 x i8>, 6) @test_vundefined_bf16m1x6(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 8 x i8>, 6) poison
//
vbfloat16m1x6_t test_vundefined_bf16m1x6() {
  return __riscv_vundefined_bf16m1x6();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 8 x i8>, 7) @test_vundefined_bf16m1x7(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 8 x i8>, 7) poison
//
vbfloat16m1x7_t test_vundefined_bf16m1x7() {
  return __riscv_vundefined_bf16m1x7();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 8 x i8>, 8) @test_vundefined_bf16m1x8(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 8 x i8>, 8) poison
//
vbfloat16m1x8_t test_vundefined_bf16m1x8() {
  return __riscv_vundefined_bf16m1x8();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 16 x i8>, 2) @test_vundefined_bf16m2x2(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 16 x i8>, 2) poison
//
vbfloat16m2x2_t test_vundefined_bf16m2x2() {
  return __riscv_vundefined_bf16m2x2();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 16 x i8>, 3) @test_vundefined_bf16m2x3(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 16 x i8>, 3) poison
//
vbfloat16m2x3_t test_vundefined_bf16m2x3() {
  return __riscv_vundefined_bf16m2x3();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 16 x i8>, 4) @test_vundefined_bf16m2x4(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 16 x i8>, 4) poison
//
vbfloat16m2x4_t test_vundefined_bf16m2x4() {
  return __riscv_vundefined_bf16m2x4();
}

// CHECK-RV64-LABEL: define dso_local target("riscv.vector.tuple", <vscale x 32 x i8>, 2) @test_vundefined_bf16m4x2(
// CHECK-RV64-SAME: ) #[[ATTR0]] {
// CHECK-RV64-NEXT:  entry:
// CHECK-RV64-NEXT:    ret target("riscv.vector.tuple", <vscale x 32 x i8>, 2) poison
//
vbfloat16m4x2_t test_vundefined_bf16m4x2() {
  return __riscv_vundefined_bf16m4x2();
}
